// Copyright 1997-1998 Omni Development, Inc.  All rights reserved.

#import "ONSocketStream.h"

#import <Foundation/Foundation.h>
#import <OmniBase/OmniBase.h>

#import "ONSocket.h"

RCS_ID("$Header: /LocalDeveloper/Source/CVS/OmniGroup/OmniNetworking/ONSocketStream.m,v 1.4 1998/01/12 06:53:52 kc Exp $")

@implementation ONSocketStream

+ streamWithSocket:(ONSocket *)aSocket;
{
    return [[[self alloc] initWithSocket:aSocket] autorelease];
}

- initWithSocket:(ONSocket *)aSocket;
{
    if (![super init])
	return nil;
    socket = [aSocket retain];
    [self clearReadBuffer];
    return self;
}

- (void)dealloc;
{
    [socket release];
    [readBuffer release];
    [super dealloc];
}


- (ONSocket *)socket;
{
    return socket;
}

- (void)setReadBuffer:(NSData *)aData;
{
    if ((NSData *)readBuffer == aData)
	return;
    [readBuffer release];
    readBuffer = [aData retain];
}

- (void)clearReadBuffer;
{
    [readBuffer release];
    readBuffer = [[NSMutableData alloc] init];
}

- (void)advanceReadBufferBy:(unsigned int)advanceAmount;
{
    NSData                     *oldReadBuffer;
    NSRange                     range;

    oldReadBuffer = readBuffer;
    range = NSMakeRange(advanceAmount, [oldReadBuffer length] - advanceAmount);
    readBuffer = [[oldReadBuffer subdataWithRange: range] mutableCopy];
    [oldReadBuffer release];
}

- (BOOL)readSocket;
{
    NSData                     *newData;
    
    newData = [socket readData];
    if (!newData)
	return NO; // End Of File
    [readBuffer appendData:newData];
    return YES;
}


- (NSString *)readLineAndAdvance:(BOOL)shouldAdvance;
{
    NSString                   *line;
    NSString                   *readBufferString;
    NSRange                     crRange, nlRange, endOfLineRange;

    endOfLineRange.length = 0;
    do {
	readBufferString =
	  [[[NSString alloc] initWithData:readBuffer
	    encoding:[self stringEncoding]] autorelease];
	nlRange = [readBufferString rangeOfString:@"\n"];
	if (nlRange.length != 0)
	    crRange = [readBufferString rangeOfString:@"\r" options:0
		       range:NSMakeRange(0, nlRange.location)];
	else
	    crRange = [readBufferString rangeOfString:@"\r"];
	if (crRange.length != 0 && nlRange.length != 0) {
	    if (crRange.location < nlRange.location) {
		endOfLineRange = crRange;
		// If the nl immediately follows cr, we found a crnl.
		if (nlRange.location - crRange.location == 1)
		    endOfLineRange.length++;
	    } else
		endOfLineRange = nlRange;
	    break;
	} else if (crRange.length != 0) {
	    endOfLineRange = crRange;
	    // if the cr is the last char in the buffer, it _might_ have a nl as the next char, so keep reading
	    if (NSMaxRange(crRange) < [readBufferString length])
		break;
	} else if (nlRange.length != 0) {
	    endOfLineRange = nlRange;
	    break;
	}
    } while ([self readSocket]);
    
    if (endOfLineRange.length != 0) {
	line = [readBufferString substringToIndex:endOfLineRange.location];
	if (shouldAdvance)
	    [self advanceReadBufferBy:NSMaxRange(endOfLineRange)];
	return line;
    }
    return nil;
}

- (NSString *)readLine;
{
    return [self readLineAndAdvance:YES];
}

- (NSString *)peekLine;
{
    return [self readLineAndAdvance:NO];
}

- (NSData *)readData;
{
    NSData                     *data;

    if ([readBuffer length] == 0) {
	if (![self readSocket])
	    return nil;
    }
    data = [[readBuffer retain] autorelease];
    [self clearReadBuffer];
    return data;
}

#ifdef ANYONE_EVER_WANTS_TO_USE_THIS
- (NSString *)readString;
{
    NSData                     *data;
    
    data = [self readData];
    if (!data)
	return nil;
    return [[[NSString alloc] initWithData:data
	     encoding:[self stringEncoding]] autorelease];
}
#endif

- (void)writeString:(NSString *)aString;
{
    [socket writeString:aString];
}

- (void)writeData:(NSData *)theData;
{
    [socket writeData:theData];
}

- (NSStringEncoding)stringEncoding;
{
    return [socket stringEncoding];
}

- (void)setStringEncoding:(NSStringEncoding)aStringEncoding;
{
    [socket setStringEncoding:aStringEncoding];
}

- (NSMutableDictionary *)debugDictionary;
{
    NSMutableDictionary        *debugDictionary;

    debugDictionary = [super debugDictionary];
    if (socket)
	[debugDictionary setObject:socket forKey:@"socket"];
    if (readBuffer)
	[debugDictionary setObject:readBuffer forKey:@"readBuffer"];

    return debugDictionary;
}

@end
